//Examples/Swing_Programs

package net.programmingcourse.coursestatistics.ui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Vector;
import java.util.logging.Logger;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.filechooser.FileNameExtensionFilter;
import javax.swing.table.DefaultTableModel;

import net.programmingcourse.coursestatistics.model.CsvFileParser;

/**
 * Main window of the program "Course Statistics"
 * 
 * @author Bernhard Steppan
 *
 */
public class MainWindow extends JFrame {
  
  private static final long serialVersionUID = 1L;
  private static final String PROGRAM_NAME = "Course Statistics";
  private static final Logger logger =
      Logger.getLogger("CourseStatistics");
  private JTable courseStatisticsTable;
  private DefaultTableModel courseStatisticsTableModel;
  
  /**
   * Main window constructor
   */
  public MainWindow() {
    super();
    logger.info("Main window is created");
    init();
  }
  
  /**
   * Creates the graphical interface
   */
  private void init() {
    logger.info("Graphical user interface is initialized");
    // Set the title of the main window:
    setTitle(PROGRAM_NAME);
    // Create menu bar:
    setJMenuBar(createMenuBar());
    // Create table component:
    add(createTableComponent(), BorderLayout.CENTER);
    // Create filter bar:
    add(createFilterBar(), BorderLayout.NORTH);
  }

  /**
   * Creates the menu bar
   * @return menueleiste
   */
  private JMenuBar createMenuBar() {
    logger.info("Menu bar is created");
    JMenuBar menuBar = new JMenuBar();
    menuBar.add(createFileMenu());
    return menuBar;
  }
  
  /**
   * Creates the file menu
   * 
   * @return the menu
   */
  private JMenu createFileMenu() {
    logger.info("File menu is created");
    JMenu fileMenu = new JMenu();
    fileMenu.setText("File");
    fileMenu.add(createFileOpenMenuCommand());
    fileMenu.add(createExitMenuCommand());
    return fileMenu;
  }

  private JMenuItem createFileOpenMenuCommand() {
    JMenuItem fileOpenMenuCommand = new JMenuItem();
    fileOpenMenuCommand.setText("Open File...");
    fileOpenMenuCommand.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        showFileOpenDialog();
      }
    });
    return fileOpenMenuCommand;
  }

  private JMenuItem createExitMenuCommand() {
    JMenuItem exitMenuCommand = new JMenuItem();
    exitMenuCommand.setText("Quit");
    exitMenuCommand.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e) {
        System.exit(0); // Exit 0 = normal program end
      }
    });
    return exitMenuCommand;
  }


  /**
   * Creates the table area in the center of the window
   * @return table area from type JScrollPane
   */
  private JScrollPane createTableComponent() {
    logger.info("Table component is created");
    // Create table for course statistics:
    courseStatisticsTableModel = new DefaultTableModel();
    // Create table with empty table model:
    courseStatisticsTable = new JTable(courseStatisticsTableModel);
    // Set font of the table header:
    courseStatisticsTable.getTableHeader().setFont(new Font("SansSerif", Font.BOLD, 12));
    return new JScrollPane(courseStatisticsTable);
  }
  
  /**
   * Created the filter bar
   * @return filterleiste
   */
  private JPanel createFilterBar() {
    logger.info("Filter bar is created");
    JLabel filterDescription = new JLabel("Filter: ");
    JTextField filterField =
      TableFilter.createFilterField(courseStatisticsTable);
    JPanel filterBar = new JPanel();
    filterBar.add(filterDescription);
    filterBar.add(filterField);
    return filterBar;
  }
  
  /**
   * Shows the file open dialog
   */
  private void showFileOpenDialog() {
    logger.info("File open dialog is called");
    JFileChooser fileOpenDialog = new JFileChooser();
    fileOpenDialog.setDialogTitle("Open Course Statistics");
    // Define file filter:
    FileNameExtensionFilter fileFilter =
      new FileNameExtensionFilter("CSV Files", "csv");
    // Set file filter:
    fileOpenDialog.setFileFilter(fileFilter);
    // Determine directory:
    File csvDirectory = new File(System.getProperty("user.dir") +
        System.getProperty("file.separator") + "csv");
    // Set current directory:
    fileOpenDialog.setCurrentDirectory(csvDirectory);
    // Show dialog:
    int returnVal = fileOpenDialog.showOpenDialog(MainWindow.this);
    // Is a file selected and OK pressed?
    if (returnVal == JFileChooser.APPROVE_OPTION) {
      File file = fileOpenDialog.getSelectedFile();
      openCsvFile(file);
    }
  }

  /**
   * Opens a CSV file and creates the table model
   * @param file
   */
  private void openCsvFile(File file) {
    String path = file.getAbsolutePath();
    String loggerMessage = "Open CSV file: " + path;
    logger.info(loggerMessage);
    try {
      CsvFileParser csvFileParser = new CsvFileParser();
      Vector<Vector<String>> records =
          csvFileParser.parseCsvFile(path);
      // Set title of columns:
      Vector<String> columnHeaders = records.elementAt(0);
      // Remove column title from the model:
      records.remove(0);
      // Update Table Model:
      courseStatisticsTableModel.setDataVector(records, columnHeaders);
      // Set the size of the table columns
      courseStatisticsTable.getColumnModel().getColumn(0).setPreferredWidth(30);
      courseStatisticsTable.getColumnModel().getColumn(1).setPreferredWidth(20);
      courseStatisticsTable.getColumnModel().getColumn(2).setPreferredWidth(20);
      courseStatisticsTable.getColumnModel().getColumn(3).setPreferredWidth(200);
      courseStatisticsTable.getColumnModel().getColumn(4).setPreferredWidth(30);
      setTitle(PROGRAM_NAME + " – " +  file.getName());
    } catch (FileNotFoundException e) {
      logger.info(e.getLocalizedMessage());
      String message =
        "The file no longer exists. Please check\n" +
        "the directory where the file is located\n" + 
        "and then try to open the file again";
      showError(message);
    } catch (IOException e) {
      logger.info(e.getLocalizedMessage());
      String message =
        "The file is corrupted. Please check\n" +
        "the file and try again afterwards\n" + 
        "to open the file.";
      showError(message);
    } catch (ArrayIndexOutOfBoundsException e) {
      logger.info(e.getLocalizedMessage());
      String message =
        "The file is empty and therefore\n" +
        "cannot be displayed.";
      showError(message);
    }
  }
  
  /**
   * Shows the error messages
   */
  private void showError(String message) {
    JOptionPane.showConfirmDialog(this, message, PROGRAM_NAME,
        JOptionPane.DEFAULT_OPTION, JOptionPane.WARNING_MESSAGE);
  }
}